! This file is part of the ITM model.
!
! Copyright 2009 University of Illinois at Urbana-Champaign
! Copyright 2011 Oregon State University, Corvallis
!
! Authors: Arturo S. Leon (Hydraulics), Nils Oberg (User interface)
!
! ITM is a free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published
! by the Free Software Foundation; either version 2.0 of the
! License, or (at your option) any later version.
! 
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
! 02110-1301, USA.

	Subroutine itm_conser_volume (current_time,delta_time)
	!This routine computes the water entered to the system and the water stored in the system. 
	use common_module
	implicit none	
	double precision Stora_new
	double precision tf,dt,current_time,delta_time
	double precision sum_temp1,sum_temp2,sum_temp3
	double precision Qinf_old,Qinf_new,Vol_enter_node,Qb
	integer i,j,R
	tf = current_time
	dt = delta_time	      
      
	!Volume stored in tunnels
	sum_temp1 = 0d0
	do j = 1,NR
		do i = 3,Nx(j)-2
			if (ISNAN(A0(j,i)))then 					
				write(98,*),'Area',A0(j,i),
     &			'pipe No',j,'cell',i
				write(98,*),'NaN is found in subr. itm_conser_volume'
				call endprog
			endif		
			if(IdFlow(j,i) == 0)then				
				!if(Area > Aref(j))then
				!	Area = Aref(j)
				!endif
				sum_temp1 = sum_temp1+A0(j,i)*dx(j)
			else				
				sum_temp1 = sum_temp1+A0(j,i)*dx(j)
			endif			
		enddo
	enddo	
	sum_temp2 = 0d0
	sum_temp3 = 0d0
	do R=1,Nnodes
		!Volume at dropshafts and junctions		
		If(BCnode(R) == 7.or.BCnode(R) == 4 .or.BCnode(R) == 24)then !reservoirs and junctions
			if (ISNAN(yres_jun_old(R)))then 				
				if (ISNAN(yres_jun_old(R)))then 					
					write(98,*),'yres_jun_old(R)',yres_jun_old(R),
     &				'Node No',R
					write(98,*),'NaN is found in subr. itm_conser_volume'
					call endprog
				endif
			endif
			sum_temp2 = sum_temp2 + Ares_junct(R)*yres_jun_old(R)	
		endif
		!Volume at reservoirs
		If(BCnode(R) == 20)then
			call get_storage(R,yres_jun_old(R),Stora_new)
			sum_temp3 = sum_temp3 + Stora_new
		endif	
	enddo
	Volume_stored_current_step = sum_temp1 + sum_temp2 + sum_temp3
	
	!Vomume that entered/exited the system at 
	!vol = initial_volume + 
	!Vol_entered_system = 0d0
	do R=1,Nnodes
		!Volume entered at dropshafts and junctions		
		If(BCnode(R) == 7.or.BCnode(R) == 4.or.BCnode(R) == 24)then !dropshafts and junctions
			!call get_curve_area(R, tf+Dt, Vol_enter_node)
			!Vol_entered_system = Vol_entered_system + Vol_enter_node
			call get_inflow(R,tf,Qinf_old)
			call get_inflow(R,tf+Dt,Qinf_new)
			Vol_entered_system = Vol_entered_system + 
     &			5.d-1*(Qinf_new+Qinf_old)*dt
		endif
				
	!Volume lost in the boundaries of the system
	!Volume lost in a Rating curve BC
	  If(BCnode(R) == 30)then !Rating curve BC
	      j = NodeID(R,1)
            if (Nodetype(R,1) == 2)then !outflowing
		        Qb = abs(Qbound(j,1))
	      elseif (Nodetype(R,1) == 1)then !inflowing
		        Qb = abs(Qbound(j,2))
	      else
	          write(98,*),'Nodetype(R,1) .ne. 1,2'
		        write(98,*),'Subr. itm_conser_volume'
		        call endprog		
	      Endif
	      Vol_lost_system = Vol_lost_system + Qb*dt
		endif  
			
		!Volume lost in a Q and h constant BC
		If(BCnode(R) == 10 .or. BCnode(R) == 11)then !10: Discharge Q constant'
	      j = NodeID(R,1)                          !11: Water depth constant'		
            if (Nodetype(R,1) == 2)then !outflowing
		        Qb = -Qbound(j,1)
	      elseif (Nodetype(R,1) == 1)then !inflowing
		        Qb = Qbound(j,2)
	      else
	          write(98,*),'Nodetype(R,1) .ne. 1,2'
		        write(98,*),'Subr. itm_conser_volume'
		        call endprog		
	      Endif
	      Vol_lost_system = Vol_lost_system + Qb*dt
		endif  
			
		!Volume lost at reservoirs	
		If(BCnode(R) == 20)then
			Vol_lost_system = Vol_lost_system + Reser_outflow(R)*dt    
		endif	
	enddo
	!Vol_entered_system = Initial_volume_stored + Vol_entered_system 	
	!Volume error of 5% may be acceptable	
	Error_volume = Vol_entered_system - Volume_stored_current_step -
     $	 Vol_lost_system
	Error_volume = 100d0*Error_volume/(0.5*(Vol_entered_system +
     &	Volume_stored_current_step + Vol_lost_system))

	!To balance conservation of volume problems
	balance_volume = abs(Vol_entered_system - 
     &	Volume_stored_current_step - Vol_lost_system)
	end subroutine
